%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo Codes For: 
%           "Level Crossing Speech Sampling and its
%          Sparsity Promoting Reconstruction using the
%          Iterative Method with Adaptive Thresholding
%                           (IMAT)"
%
% References:
%
% [1] M. Boloursaz, E. S. Farahani, N. Salarieh, and F. Marvasti, Level Crossing Speech Sampling and its Sparsity Promoting Reconstruction 
%     using the Iterative Method with Adaptive Thresholding (IMAT), Submitted to IEEE Transactions on Audio, Speech and Language Processing, March 2015
% [2] Farokh Marvasti et al., "A Unified Approach to Sparse Signal Processing," EURASIP Journal on Advances in Signal Processing,2012
%
% Written By: Nikan Salarieh
%             Mahdi Boloursaz Mashhadi
%
% Affiliation: 
%          Advanced Communications Research Institute (ACRI)
% Electrical Engineering Department, Sharif University of Technology
%                              Tehran, Iran
%
% Description:
%
% This demo code goes with submission [1]. It compares the performance of different sparsity promoting (IMAT, IMATI, OMP and LASSO) and 
% LowPass based (Iterative and IterativeI) algorithms for speech signal reconstruction from Adaptive Level Crossing (ALC) samples. It proves 
% superior performance of the proposed IMAT and IMATI techniques in terms of avaerage reconstruction SNR and Success Rate (SR).
%
% For any problems, contact me at Nikan_fds@yahoo.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
%%                             Initialization
close all
clear all
clc
%%                          Simulation Parameters  
r = 50;                        % Random (Missing) Sampling Rate (Percent)
frameNum=331;                  % Number of Frames
% IMAT Parameters
alpha_IMAT = 0.009;            % Exponential Threshold Reduction Factor
beta_IMAT = 0.2;               % Initial Threshold
landa_IMAT = 1.2;              % Relaxation Parameter
IterNum_IMAT = 600;            % Number of Algorithm Iterations
% IMATI Parameters
alpha_IMATI = 0.4;            % Exponential Threshold Reduction Factor
beta_IMATI = 0.01;             % Initial Threshold
landa_IMATI = 1.08;             % Relaxation Parameter
IterNum_IMATI = 100;           % Number of Algorithm Iterations
% Iterative Parameters
landa_Iterative = 2;           % Relaxation Parameter
IterNum_Iterative = 1000;      % Number of Algorithm Iterations 
% IterativeI Parameters
landa_IterativeI = 2;          % Relaxation Parameter
IterNum_IterativeI = 100;     % Number of Algorithm Iterations
% Lasso Parameter
landa_LASSO = 0.003;          % Regularization Parameter
%%                              Main Body
data=wavread('Voice.wav');     % Loading Voice
for f=1:frameNum
    
data0=data((f-1)*960+1:f*960); % Input Frame
s=data0;
m=length(data0);

Energy_frame(f)=sum(data0.^2); % Silence Detection
if Energy_frame(f)<4.9
    snrIMAT_mat(f)=0;
    snrIMATI_mat(f)=0;
    snrIterative_mat(f)=0;
    snrIterativeI_mat(f)=0;
    snromp_mat(f)=0;
    snrLasso_mat(f)=0;
    continue;
end
%%                   ALC Sampling for Sparse Assumption
d=(max(data0)-min(data0))/5;   % Distance between Levels
[a,samp_num,n]=ALCSampling(s,d,m);         
turn=0;
while (n<r-1 || n>r+1)& turn<10
if n>r+5
    d=d+.003;
elseif n<r-1
    d=.5*d;
end
[a,samp_num,n]=ALCSampling(s,d,m);
turn=turn+1;
end
%%
msk=zeros(m,1);
for kk=1:length(samp_num)
    msk(samp_num(kk))=1;
end
%%                      Reconstruction by IMAT
c=a';
data1=c;
        
        LCSig=a';
        x=LCSig; 
        xnew=0;
        snr_IMAT(1)=10*log10(sum(data0.^2)/sum((data0-x).^2));
        for i=2:IterNum_IMAT 
            tx=dct(x);                           % Switching to the Sparse Domain
            th=beta_IMAT*exp(-alpha_IMAT*(i-2)); % Adapting the Threshold
            ttmsk=zeros(length(data0),1);
            ttmsk(abs(tx)>th)=1;
            x=idct(ttmsk.*tx);                   
            xnew=landa_IMAT*LCSig+(1-landa_IMAT*msk).*x; % Recursion Formula
            snr_IMAT(i)=10*log10(sum(data0.^2)/sum((data0-xnew).^2));
            x=xnew;
        end
snrIMAT_mat(f)=max(snr_IMAT);
%%                      Reconstruction by IMATI
c=a';
aaa=c;
data2=aaa;
ddddd=data2;
%Intepolation
            c=0;
            for ii=1:m
                if (ddddd(ii)==0 && data0(ii)~=0)
                    data2(ii)=c;
                else
                    data2(ii)=ddddd(ii);
                    c=ddddd(ii);
                end
            end
data_int1=data2;

x=zeros(1,m);
%IMATI reconstruction
data000=data2;
for i=1:IterNum_IMATI
    
%Intepolation
xx=x;
            cc=0;
            for ii=1:m
                if (ddddd(ii)==0 && data0(ii)~=0)
                    xx(ii)=cc;
                else
                    xx(ii)=x(ii);
                    cc=x(ii);
                end
            end
    data1=x+landa_IMATI*(data000'-xx);     % Recursion Formula
    x=dct(data1);
   xxxx=x;
   th=beta_IMATI*exp(-alpha_IMATI*(i-1));  % Adapting the Threshold
   
   ppp=1;
   for j=1:m
        if(abs(x(j))<th)
           x(j)=0;
           ppp=ppp+1;
        end
    end
    
       x=idct(x);

       snr_IMATI(i)=(10*log10([sum(data0'.^2)]/[sum((x-data0').^2)]));
 
end
snrIMATI_mat(f)=max(snr_IMATI);
%%                      Reconstruction by OMP
D=dctmtx(length(data0));
DI=D';
A=DI(find(msk),:);
LCSig=a';
y=LCSig(find(msk));
[solsomp, iters, activationHist] = SolveOMP(A, y, m);
xomp=DI*solsomp;
snromp=10*log10(sum(data0.^2)/sum((data0-xomp).^2));
snromp_mat(f)=snromp;
%%                      Reconstruction by LASSO
[solsLasso, iters, activationHist] = SolveLasso(A, y, m,'lasso',10000,landa_LASSO);
xLasso=DI*solsLasso;
snrLasso=10*log10(sum(data0.^2)/sum((data0-xLasso).^2));
snrLasso_mat(f)=snrLasso;
%%                   ALC Sampling for LowPass Assumption
s=data0;
%LowPass filter
BW=fix(960*(.01*r));
LPmsk=[ones(BW,1);zeros(length(s)-BW,1)];
V=LPmsk.*dct(s);
s=idct(V);

d=(max(data0)-min(data0))/5;
[a,samp_num,n]=ALCSampling(s,d,m);         
turn=0;
while (n<r-1 || n>r+5)& turn<10
if n>r+5
    d=d+.003;
elseif n<r-1
    d=.5*d;
end
[a,samp_num,n]=ALCSampling(s,d,m);
turn=turn+1;
end

msk=zeros(m,1);
for kk=1:length(samp_num)
    msk(samp_num(kk))=1;
end
%%                  Reconstruction by Iterative
c=a';

dddddd=c';
data1=c';
%LowPass filter
BW=fix(960/2*(.01*r));
LPmsk=[ones(BW,1);zeros(length(data1)-BW,1)];
V=LPmsk'.*dct(data1);
y=idct(V);
x=y;


c=ones(1,m);
for ii=1:m
if (dddddd(ii)==0)
    c(ii)=0;
end
end

        for i=1:IterNum_Iterative
            data1=x+landa_Iterative*(y-idct(LPmsk'.*dct(c.*x)));
            
            x=data1;
       

            snr_Iterative(i)=(10*log10([sum(data0.^2)]/[sum((data1'-data0).^2)]));
        end
snrIterative_mat(f)=max(snr_Iterative);
%%                  Reconstruction by IterativeI
c=a';
ddd=c;
%Interpolation
            c=0;
            for ii=1:m
                if ddd(ii)==0
                    data00(ii)=c;
                else
                    data00(ii)=ddd(ii);
                    c=ddd(ii);
                end
            end

x=zeros(1,m);
%IterativeI reconstruction
data11=data00;
%LowPass filter
BW=fix(960*(.01*r));
LPmsk=[ones(BW,1);zeros(length(data11)-BW,1)];
V=LPmsk.*dct(data11');
y=idct(V);
x=y;


     for i=1:IterNum_IterativeI
            %Interpolation
xxx=x;
            cc=0;
            for ii=1:m
                if ddd(ii)==0
                    xxx(ii)=cc;
                else
                    xxx(ii)=x(ii);
                    cc=x(ii);
                end
            end
            %end
        data11=x+landa_IterativeI*(y-idct(LPmsk.*dct(xxx)));
        x=data11;
        snr_IterativeI(i)=10*log10([sum(data0.^2)]/[sum((data11-data0).^2)]);
     end
snrIterativeI_mat(f)=max(snr_IterativeI);
end
%%                         Simulation Results
success_th=25;      % SNRs achieved above this threshold are considered successful reconstruction 
% Results of IMAT
v=0;
w=0;
for f=1:frameNum
    if snrIMAT_mat(f)==0
        continue;
    end
    
    if snrIMAT_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IMAT_successrate=v/(v+w)
IMAT_snr=mean(snrIMAT_mat(find(snrIMAT_mat>0)))
% Results of IMATI
v=0;
w=0;
for f=1:frameNum
    if snrIMATI_mat(f)==0
        continue;
    end
    if snrIMATI_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IMATI_successrate=v/(v+w)
IMATI_snr=mean(snrIMATI_mat(find(snrIMATI_mat>0)))
% Results of Iterative
v=0;
w=0;
for f=1:frameNum
    if snrIterative_mat(f)==0
        continue;
    end
    if snrIterative_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
Iterative_successrate=v/(v+w)
Iterative_snr=mean(snrIterative_mat(find(snrIterative_mat>0)))
% Results of IterativeI
v=0;
w=0;
for f=1:frameNum
    if snrIterativeI_mat(f)==0
        continue;
    end
    if snrIterativeI_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IterativeI_successrate=v/(v+w)
IterativeI_snr=mean(snrIterativeI_mat(find(snrIterativeI_mat>0)))
% Results of OMP
v=0;
w=0;
for f=1:frameNum
    if snromp_mat(f)==0
        continue;
    end
    if snromp_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
OMP_successrate=v/(v+w)
OMP_snr=mean(snromp_mat(find(snromp_mat>0)))
% Results of LASSO
v=0;
w=0;
for f=1:331
    if snrLasso_mat(f)==0
        continue;
    end
    if snrLasso_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
LASSO_successrate=v/(v+w)
LASSO_snr=mean(snrLasso_mat(find(snrLasso_mat>0)))
toc