%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo Codes For: 
%           "Level Crossing Speech Sampling and its
%          Sparsity Promoting Reconstruction using the
%          Iterative Method with Adaptive Thresholding
%                           (IMAT)"
%
% References:
%
% [1] M. Boloursaz, E. S. Farahani, N. Salarieh, and F. Marvasti, Level Crossing Speech Sampling and its Sparsity Promoting Reconstruction 
%     using the Iterative Method with Adaptive Thresholding (IMAT), Submitted to IEEE Transactions on Audio, Speech and Language Processing, March 2015
% [2] Farokh Marvasti et al., "A Unified Approach to Sparse Signal Processing," EURASIP Journal on Advances in Signal Processing,2012
%
% Written By: Ehsan Shahrabi Farahani
%             Mahdi Boloursaz Mashhadi
%
% Affiliation: 
%          Advanced Communications Research Institute (ACRI)
% Electrical Engineering Department, Sharif University of Technology
%                              Tehran, Iran
%
% Description:
%
% This demo code goes with submission [1]. It compares the performance of different sparsity promoting (IMAT, IMATI, OMP and LASSO) and 
% LowPass based (Iterative and IterativeI) algorithms for speech signal reconstruction from Level Crossing (LC) samples. It proves 
% superior performance of the proposed IMAT and IMATI techniques in terms of avaerage reconstruction SNR and Success Rate (SR).
%
% For any problems, contact me at ehsan.shahrabi.f@gmail.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
%%                             Initialization
close all
clear all
clc
%%                          Simulation Parameters  
r = 50;                       % Random (Missing) Sampling Rate (Percent)
frameNum=331;                 % Number of Frames
% IMAT Parameters
alpha_IMAT = 0.03;            % Exponential Threshold Reduction Factor
beta_IMAT = 0.5;              % Initial Threshold
landa_IMAT = 1.3;             % Relaxation Parameter
IterNum_IMAT = 500;           % Number of Algorithm Interpolations
% IMATI Parameters
alpha_IMATI = 0.3;            % Exponential Threshold Reduction Factor
beta_IMATI = 0.04;            % Initial Threshold
landa_IMATI = 0.9;            % Relaxation Parameter
IterNum_IMATI = 500;          % Number of Algorithm Interpolations
% Iterative Parameters
landa_Iterative = 2;        % Relaxation Parameter
IterNum_Iterative = 1000;     % Number of Algorithm Interpolations 
% IterativeI Parameters
landa_IterativeI = .006;      % Relaxation Parameter
IterNum_IterativeI = 500;     % Number of Algorithm Interpolations
%%                              Main Body
data=wavread('Voice.wav');    % Loading Voice
maxx=max(abs(data));
for f=1:frameNum
    
data0=data((f-1)*960+1:f*960); % Input Frame
s=data0;
m=length(data0);

Energy_frame(f)=sum(data0.^2); % Silence Detection
if Energy_frame(f)<4.9
    snrIMAT_mat(f)=0;
    snrIMATI_mat(f)=0;
    snrIterative_mat(f)=0;
    snrIterativeI_mat(f)=0;
    snromp_mat(f)=0;
    snrLasso_mat(f)=0;
    continue;
end
%%                   LC Sampling for Sparse Assumption
force=1;
L=50;
while force==1 %enforcing the determined sampling percent
    data01=data0; 
%Level measures
q=2*maxx/L;
for iiiii=1:L
g(iiiii)=-maxx+q/2+(iiiii-1)*q;
end
%Level crossing
m=length(data0);
zzzzzz=zeros(1,m);
z=zeros(m,L);
for i=1:m
    for j=1:L
       z(i,j)=data0(i)-g(j);    
    end
end

kkk=0;
for j=1:m-1
        fff=0;
        i=1;
       while fff==0 && i<L+1
        if z(j,i)*z(j+1,i)>0
            
        else
            zzzzzz(j)=1;
            fff=1;
            i=L+5;
            kkk=kkk+1; %for counting 
        end
        i=i+1;
       end
end

data01=zzzzzz.*data0';
if f==0
data123=data01;
end
percent_of_sampling(f+1)=kkk/m*100;

if percent_of_sampling(f+1)>50
    force=2;
else
    L=L+1;
end
end
ddddd=data01;
%%
msk=zzzzzz;
%%                      Reconstruction by IMAT
c=ddddd;
data1=c;
        
        LCSig=c;
        x=LCSig; 
        xnew=0;
        snr(1)=10*log10(sum(data0.^2)/sum((data0-x').^2));
        for i=1:IterNum_IMAT 
            tx=dct(x);                           % Switching to the Sparse Domain
            th=beta_IMAT*exp(-alpha_IMAT*(i-1)); % Adapting the Threshold
            ttmsk=zeros(length(data0),1);
            ttmsk(abs(tx)>th)=1;
            x=idct(ttmsk'.*tx);                   
            xnew=landa_IMAT*LCSig+(1-landa_IMAT*msk).*x; % Recursion Formula
            snr(i+1)=10*log10(sum(data0.^2)/sum((data0-xnew').^2));
            x=xnew;
        end
snrIMAT_mat(f)=max(snr);
%%                      Reconstruction by IMATI
c=ddddd;
aaa=c;
data2=aaa;
ddddd1=data2;
%Intepolation
            c=0;
            for ii=1:m
                if (ddddd1(ii)==0 && data0(ii)~=0)
                    data2(ii)=c;
                else
                    data2(ii)=ddddd1(ii);
                    c=ddddd1(ii);
                end
            end
data_int1=data2;

x=zeros(1,m);
%IMATI reconstruction
data000=data2;
for i=1:IterNum_IMATI
    
%Interpolation
xx=x;
            cc=0;
            for ii=1:m
                if (ddddd1(ii)==0 && data0(ii)~=0)
                    xx(ii)=cc;
                else
                    xx(ii)=x(ii);
                    cc=x(ii);
                end
            end
    data1=x+landa_IMATI*(data000-xx);     % Recursion Formula
    x=dct(data1);
   xxxx=x;
   th=beta_IMATI*exp(-alpha_IMATI*(i-1));  % Adapting the Threshold
   
   ppp=1;
   for j=1:m
        if(abs(x(j))<th)
           x(j)=0;
           ppp=ppp+1;
        end
    end
    
       x=idct(x);

       snr(i)=(10*log10([sum(data0'.^2)]/[sum((x-data0').^2)]));
 
end
snrIMATI_mat(f)=max(snr);
%%                      Reconstruction by OMP
D=dctmtx(length(data0));
DI=D';
A=DI(find(msk),:);
LCSig=ddddd;
y=LCSig(find(msk));
[solsomp, iters, activationHist] = SolveOMP(A, y', m, 70);
xomp=DI*solsomp;
snromp=10*log10(sum(data0.^2)/sum((data0-xomp).^2));
snromp_mat(f)=snromp;
%%                      Reconstruction by LASSO
[solsLasso, iters, activationHist] = SolveLasso(A, y', m,'lasso',10000,.006);
xLasso=DI*solsLasso;
snrLasso=10*log10(sum(data0.^2)/sum((data0-xLasso).^2));
snrLasso_mat(f)=snrLasso;
%%                   LC Sampling for LowPass Assumption
BW=fix(960*(.01*r));
LPmsk=[ones(BW,1);zeros(length(s)-BW,1)];
V=LPmsk.*dct(s);
s=idct(V);

while force==1 %just forcing to gathering the determined sampling percent
    data01=data0; 
%Level measures
q=2*maxx/L;
for iiiii=1:L
g(iiiii)=-maxx+q/2+(iiiii-1)*q;
end
%Level crossing
m=length(data0);
zzzzzz=zeros(1,m);
z=zeros(m,L);
for i=1:m
    for j=1:L
       z(i,j)=data0(i)-g(j);    
    end
end

kkk=0;
for j=1:m-1
        fff=0;
        i=1;
       while fff==0 && i<L+1
        if z(j,i)*z(j+1,i)>0
            
        else
            zzzzzz(j)=1;
            fff=1;
            i=L+5;
            kkk=kkk+1; %for counting 
        end
        i=i+1;
       end
end

data01=zzzzzz.*data0;
if f==0
data123=data01;
end
percent_of_sampling(f+1)=kkk/m*100;

if percent_of_sampling(f+1)>50
    force=2;
else
    L=L+1;
    levelnumber(perper)=L;
    perper=perper+1;
end
end
ddddd11=data01;

msk=zzzzzz;
%%                  Reconstruction by Iterative
c=ddddd11;

dddddd=c';
data1=c';
%LowPass filter
BW=fix(960/2*(.01*r));
LPmsk=[ones(BW,1);zeros(length(data1)-BW,1)];
V=LPmsk.*dct(data1);
y=idct(V);
x=y;


c=ones(1,m);
for ii=1:m
if (dddddd(ii)==0)
    c(ii)=0;
end
end

        for i=1:IterNum_Iterative
            data1=x+landa_Iterative*(y-idct(LPmsk.*dct(c'.*x)));
            
            x=data1;
       

            snr_Iterative(i)=(10*log10([sum(data0.^2)]/[sum((data1-data0).^2)]));
        end
snrIterative_mat(f)=max(snr_Iterative);
%%                  Reconstruction by IterativeI
c=ddddd11;
ddd=c;
%Intepolation
            c=0;
            for ii=1:m
                if ddd(ii)==0
                    data00(ii)=c;
                else
                    data00(ii)=ddd(ii);
                    c=ddd(ii);
                end
            end

x=zeros(1,m);
%IterativeI reconstruction
data11=data00;
%LowPass filter
BW=fix(960*(.01*r));
LPmsk=[ones(BW,1);zeros(length(data11)-BW,1)];
V=LPmsk.*dct(data11');
y=idct(V);
x=y;


     for i=1:IterNum_IterativeI
            %Interpolation
xxx=x;
            cc=0;
            for ii=1:m
                if ddd(ii)==0
                    xxx(ii)=cc;
                else
                    xxx(ii)=x(ii);
                    cc=x(ii);
                end
            end
            %end
        data11=x+landa_IterativeI*(y-idct(LPmsk.*dct(xxx)));
        x=data11;
        snr_IterativeI(i)=10*log10([sum(data0.^2)]/[sum((data11-data0).^2)]);
     end
snrIterativeI_mat(f)=max(snr_IterativeI);
end
%%                         Simulation Results
success_th=25;
% Results of IMAT
v=0;
w=0;
for f=1:frameNum
    if snrIMAT_mat(f)==0
        continue;
    end
    
    if snrIMAT_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IMAT_successrate=v/(v+w)
IMAT_snr=mean(snrIMAT_mat(find(snrIMAT_mat>0)))
% Results of IMATI
v=0;
w=0;
for f=1:frameNum
    if snrIMATI_mat(f)==0
        continue;
    end
    if snrIMATI_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IMATI_successrate=v/(v+w)
IMATI_snr=mean(snrIMATI_mat(find(snrIMATI_mat>0)))
% Results of Iterative
v=0;
w=0;
for f=1:frameNum
    if snrIterative_mat(f)==0
        continue;
    end
    if snrIterative_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
Iterative_successrate=v/(v+w)
Iterative_snr=mean(snrIterative_mat(find(snrIterative_mat>0)))
% Results of IterativeI
v=0;
w=0;
for f=1:frameNum
    if snrIterative_mat(f)==0
        continue;
    end
    if snrIterativeI_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
IterativeI_successrate=v/(v+w)
IterativeI_snr=mean(snrIterativeI_mat(find(snrIterativeI_mat>0)))
% Results of OMP
v=0;
w=0;
for f=1:frameNum
    if snromp_mat(f)==0
        continue;
    end
    if snromp_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
OMP_successrate=v/(v+w)
OMP_snr=mean(snromp_mat(find(snromp_mat>0)))
% Results of LASSO
v=0;
w=0;
for f=1:331
    if snrLasso_mat(f)==0
        continue;
    end
    if snrLasso_mat(f)>=success_th
        v=v+1;
    else w=w+1;
    end
end
LASSO_successrate=v/(v+w)
LASSO_snr=mean(snrLasso_mat(find(snrLasso_mat>0)))
toc